@@ -281,38 +281,34 @@ Accepted (4 linkages, 4 with no P.P. violations) at stage 1
 
 */
 
-static int STAT_N_disjuncts;	  /* keeping statistics */
-static int STAT_calls_to_equality_test;
-
-static void init_LT(Sentence sent) {
+static void init_LT(Sentence sent)
+{
 	sent->and_data.LT_bound = 20;
 	sent->and_data.LT_size = 0;
 	sent->and_data.label_table =
 		(Disjunct **) xalloc(sent->and_data.LT_bound * sizeof(Disjunct *));
 }
 
-static void grow_LT(Sentence sent) {
-	space_in_use -= sent->and_data.LT_bound * sizeof(Disjunct *);
+static void grow_LT(Sentence sent)
+{
+	size_t oldsize = sent->and_data.LT_bound * sizeof(Disjunct *);
 	sent->and_data.LT_bound = (3*sent->and_data.LT_bound)/2;
 	sent->and_data.label_table =
-		(Disjunct **) realloc((void *)sent->and_data.label_table,
-							  sent->and_data.LT_bound * sizeof(Disjunct *));
-	space_in_use += sent->and_data.LT_bound * sizeof(Disjunct *);
-	if (space_in_use > max_space_in_use) max_space_in_use = space_in_use;
-	if (sent->and_data.label_table == NULL) {
-		printf("Ran out of space reallocing the label table\n");
-		exit(1);
-	}
+		(Disjunct **) xrealloc(sent->and_data.label_table,
+	                     oldsize,
+		                  sent->and_data.LT_bound * sizeof(Disjunct *));
 }
 
-static void init_HT(Sentence sent) {
+static void init_HT(Sentence sent)
+{
 	int i;
 	for (i=0; i<HT_SIZE; i++) {
 		sent->and_data.hash_table[i] = NULL;
 	}
 }
 
-static void free_HT(Sentence sent) {
+static void free_HT(Sentence sent)
+{
 	int i;
 	Label_node * la, * la1;
 	for (i=0; i<HT_SIZE; i++) {
@@ -324,7 +320,8 @@ static void free_HT(Sentence sent) {
 	}
 }
 
-static void free_LT(Sentence sent) {
+static void free_LT(Sentence sent)
+{
 	int i;
 	for (i=0; i<sent->and_data.LT_size; i++) {
 		free_disjuncts(sent->and_data.label_table[i]);
@@ -336,12 +333,14 @@ static void free_LT(Sentence sent) {
 	sent->and_data.label_table = NULL;
 }
 
-void free_AND_tables(Sentence sent) {
+void free_AND_tables(Sentence sent)
+{
 	free_LT(sent);
 	free_HT(sent);
 }
 
-void initialize_conjunction_tables(Sentence sent) {
+void initialize_conjunction_tables(Sentence sent)
+{
 	int i;
 	sent->and_data.LT_bound = 0;
 	sent->and_data.LT_size = 0;
@@ -382,20 +381,22 @@ static int and_hash_disjunct(Disjunct *d)
 }
 
 
-static int is_appropriate(Sentence sent, Disjunct * d) {
-/* returns TRUE if the disjunct is appropriate to be made into fat links.
-   Check here that the connectors are from some small set.
-   This will disallow, for example "the and their dog ran".
-*/
+/**
+ * Returns TRUE if the disjunct is appropriate to be made into fat links.
+ * Check here that the connectors are from some small set.
+ * This will disallow, for example "the and their dog ran".
+ */
+static int is_appropriate(Sentence sent, Disjunct * d)
+{
 	Connector * c;
 
 	if (sent->dict->andable_connector_set == NULL) return TRUE;
 	/* if no set, then everything is considered andable */
 	for (c = d->right; c!=NULL; c=c->next) {
-		if (!match_in_connector_set(sent->dict->andable_connector_set, c, '+')) return FALSE;
+		if (!match_in_connector_set(sent, sent->dict->andable_connector_set, c, '+')) return FALSE;
 	}
 	for (c = d->left; c!=NULL; c=c->next) {
-		if (!match_in_connector_set(sent->dict->andable_connector_set, c, '-')) return FALSE;
+		if (!match_in_connector_set(sent, sent->dict->andable_connector_set, c, '-')) return FALSE;
 	}
 	return TRUE;
 }
@@ -488,24 +489,27 @@ const char * intersect_strings(Sentence sent, const char * s, const char * t)
 	}
 }
 
-static int connectors_equal_AND(Connector *c1, Connector *c2) {
-/* Two connectors are said to be equal if they are of the same type
-   (defined above), they have the same multi field, and they have
-   exactly the same connectors (including lower case chars).
-   (priorities ignored).
-*/
+/**
+ * Two connectors are said to be equal if they are of the same type
+ * (defined above), they have the same multi field, and they have
+ *  exactly the same connectors (including lower case chars).
+ * (priorities ignored).
+ */
+static int connectors_equal_AND(Connector *c1, Connector *c2)
+{
 	return (c1->label == c2->label) &&
 		   (c1->multi == c2->multi) &&
 		   (strcmp(c1->string, c2->string) == 0);
 }
 
-
-static int disjuncts_equal_AND(Disjunct * d1, Disjunct * d2) {
-/* Return true if the disjuncts are equal (ignoring priority fields)
-   and the string of the disjunct.
-*/
+/**
+ * Return true if the disjuncts are equal (ignoring priority fields)
+ * and the string of the disjunct.
+ */
+static int disjuncts_equal_AND(Sentence sent, Disjunct * d1, Disjunct * d2)
+{
 	Connector *e1, *e2;
-	STAT_calls_to_equality_test++;
+	sent->and_data.STAT_calls_to_equality_test++;
 	e1 = d1->left;
 	e2 = d2->left;
 	while((e1!=NULL) && (e2!=NULL)) {
@@ -525,12 +529,13 @@ static int disjuncts_equal_AND(Disjunct * d1, Disjunct * d2) {
 	return TRUE;
 }
 
+/**
+ * Create a new disjunct that is the GCD of d1 and d2.
+ * It assumes that the disjuncts are of the same type, so the
+ * GCD will not be empty.
+ */
 static Disjunct * intersect_disjuncts(Sentence sent, Disjunct * d1, Disjunct * d2)
 {
-/* Create a new disjunct that is the GCD of d1 and d2.
-   It assumes that the disjuncts are of the same type, so the
-   GCD will not be empty.
-*/
 	Disjunct * d;
 	Connector *c1, *c2, *c;
 	d = copy_disjunct(d1);
@@ -553,31 +558,36 @@ static Disjunct * intersect_disjuncts(Sentence sent, Disjunct * d1, Disjunct * d
 	return d;
 }
 
-static void put_disjunct_into_table(Sentence sent, Disjunct *d) {
-/* (1) look for the given disjunct in the table structures
-	   if it's already in the table structures, do nothing
-   (2) otherwise make a copy of it, and put it into the table structures
-   (3) also put all of the GCDs of this disjunct with all of the
-	   other matching disjuncts into the table.
-
-   The costs are set to zero.
-   Note that this has no effect on disjunct d.
-*/
+/**
+ * (1) look for the given disjunct in the table structures
+ *     if it's already in the table structures, do nothing
+ * (2) otherwise make a copy of it, and put it into the table structures
+ * (3) also put all of the GCDs of this disjunct with all of the
+ *     other matching disjuncts into the table.
+ *
+ * The costs are set to zero.
+ * Note that this has no effect on disjunct d.
+ */
+static void put_disjunct_into_table(Sentence sent, Disjunct *d)
+{
 	Disjunct *d1=NULL, *d2, *di, *d_copy;
 	Label_node * lp;
 	int h, k;
 
 	h = and_hash_disjunct(d);
 
-	for (lp = sent->and_data.hash_table[h]; lp != NULL; lp = lp->next) {
+	for (lp = sent->and_data.hash_table[h]; lp != NULL; lp = lp->next)
+	{
 		d1 = sent->and_data.label_table[lp->label];
 		if (disjunct_types_equal(d,d1)) break;
 	}
-	if (lp != NULL) {
+	if (lp != NULL)
+	{
 		/* there is already a label for disjuncts of this type */
 		/* d1 points to the list of disjuncts of this type already there */
-		while(d1 != NULL) {
-			if (disjuncts_equal_AND(d1, d)) return;
+		while(d1 != NULL)
+		{
+			if (disjuncts_equal_AND(sent, d1, d)) return;
 			d1 = d1->next;
 		}
 		/* now we must put the d disjunct in there, and all of the GCDs of
@@ -612,10 +622,10 @@ static void put_disjunct_into_table(Sentence sent, Disjunct *d) {
 		for (;d2 != NULL; d2 = di) {
 			di = d2->next;
 			for (d1 = sent->and_data.label_table[k]; d1 != NULL; d1 = d1->next) {
-				if (disjuncts_equal_AND(d1, d2)) break;
+				if (disjuncts_equal_AND(sent, d1, d2)) break;
 			}
 			if (d1 == NULL) {
-				STAT_N_disjuncts++;
+				sent->and_data.STAT_N_disjuncts++;
 				d2->next = sent->and_data.label_table[k];
 				sent->and_data.label_table[k] = d2;
 			} else {
@@ -635,19 +645,20 @@ static void put_disjunct_into_table(Sentence sent, Disjunct *d) {
 		lp->label = sent->and_data.LT_size;
 		sent->and_data.label_table[sent->and_data.LT_size] = d_copy;
 		sent->and_data.LT_size++;
-		STAT_N_disjuncts++;
+		sent->and_data.STAT_N_disjuncts++;
 	}
 }
 
-
-static void extract_all_fat_links(Sentence sent, Disjunct * d) {
-/*  A sub disjuct of d is any disjunct obtained by killing the tail
-    of either connector list at any point.
-    Here we go through each sub-disjunct of d, and put it into our
-    table data structure.
-
-    The function has no side effects on d.
+/**
+ * A sub disjuct of d is any disjunct obtained by killing the tail
+ *  of either connector list at any point.
+ *  Here we go through each sub-disjunct of d, and put it into our
+ *  table data structure.
+ *
+ *  The function has no side effects on d.
  */
+static void extract_all_fat_links(Sentence sent, Disjunct * d)
+{
 	Connector * cl, * cr, *tl, *tr;
 	tl = d->left;
 	d->left = NULL;
@@ -683,7 +694,7 @@ static void extract_all_fat_links(Sentence sent, Disjunct * d) {
 	}
 }
 
-/** 
+/**
  * put the next len characters from c->string (skipping upper
  * case ones) into s.  If there are fewer than this, pad with '*'s.
  * Then put in a character for the multi match bit of c.
@@ -772,17 +783,20 @@ static void compute_matchers_for_a_label(Sentence sent, int k)
 	xfree((char *)lengths, N_connectors*sizeof(int));
 }
 
-void build_conjunction_tables(Sentence sent) {
-/* Goes through the entire sentence and builds the fat link tables
-   for all the disjuncts of all the words.
-*/
+/**
+ * Goes through the entire sentence and builds the fat link tables
+ * for all the disjuncts of all the words.
+ */
+void build_conjunction_tables(Sentence sent)
+{
 	int w;
 	int k;
 	Disjunct * d;
 
 	init_HT(sent);
 	init_LT(sent);
-	STAT_N_disjuncts = STAT_calls_to_equality_test = 0;
+	sent->and_data.STAT_N_disjuncts = 0;
+	sent->and_data.STAT_calls_to_equality_test = 0;
 
 	for (w=0; w<sent->length; w++) {
 		for (d=sent->word[w].d; d!=NULL; d=d->next) {
@@ -795,14 +809,15 @@ void build_conjunction_tables(Sentence sent) {
 	}
 }
 
-void print_AND_statistics(Sentence sent) {
+void print_AND_statistics(Sentence sent)
+{
 	printf("Number of disjunct types (labels): %d\n", sent->and_data.LT_size);
-	printf("Number of disjuncts in the table: %d\n", STAT_N_disjuncts);
+	printf("Number of disjuncts in the table: %d\n", sent->and_data.STAT_N_disjuncts);
 	if (sent->and_data.LT_size != 0) {
 	  printf("average list length: %f\n",
-			 (float)STAT_N_disjuncts/sent->and_data.LT_size);
+			 (float)sent->and_data.STAT_N_disjuncts/sent->and_data.LT_size);
 	}
-	printf("Number of equality tests: %d\n", STAT_calls_to_equality_test);
+	printf("Number of equality tests: %d\n", sent->and_data.STAT_calls_to_equality_test);
 }
 
 /**
@@ -824,7 +839,7 @@ static void connector_for_disjunct(Sentence  sent, Disjunct * d, Connector * c)
 	assert(lp != NULL, "A disjunct I inserted was not there. (1)");
 
 	while(d1 != NULL) {
-		if (disjuncts_equal_AND(d1, d)) break;
+		if (disjuncts_equal_AND(sent, d1, d)) break;
 		d1 = d1->next;
 	}
 
@@ -928,10 +943,12 @@ static Disjunct * build_fat_link_substitutions(Sentence sent, Disjunct *d)
 	return d_list;
 }
 
-Disjunct * explode_disjunct_list(Sentence sent, Disjunct *d) {
-/*  This is basically a "map" function for build_fat_link_substitutions.
-    It's applied to the disjuncts for all regular words of the sentence.
-*/
+/**
+ * This is basically a "map" function for build_fat_link_substitutions.
+ * It's applied to the disjuncts for all regular words of the sentence.
+ */
+Disjunct * explode_disjunct_list(Sentence sent, Disjunct *d)
+{
    Disjunct *d1;
 
    d1 = NULL;
@@ -942,12 +959,14 @@ Disjunct * explode_disjunct_list(Sentence sent, Disjunct *d) {
    return d1;
 }
 
-Disjunct * build_COMMA_disjunct_list(Sentence sent) {
-/*  Builds and returns a disjunct list for the comma.  These are the
-    disjuncts that are used when "," operates in conjunction with "and".
-    Does not deal with the ", and" issue, nor the other uses
-    of comma.
-*/
+/**
+ * Builds and returns a disjunct list for the comma.  These are the
+ * disjuncts that are used when "," operates in conjunction with "and".
+ * Does not deal with the ", and" issue, nor the other uses
+ * of comma.
+ */
+Disjunct * build_COMMA_disjunct_list(Sentence sent)
+{
 	int lab;
 	Disjunct *d1, *d2, *d, work_disjunct, *wd;
 	Connector work_connector1, work_connector2, *c1, *c2;
@@ -982,13 +1001,15 @@ Disjunct * build_COMMA_disjunct_list(Sentence sent) {
 	return d1;
 }
 
-Disjunct * build_AND_disjunct_list(Sentence sent, char * s) {
- /* Builds and returns a disjunct list for "and", "or" and "nor" */
- /* for each disjunct in the label_table, we build three disjuncts */
- /* this means that "Danny and Tycho and Billy" will be parsable in */
- /* two ways.  I don't know an easy way to avoid this */
- /* the string is either "and", or "or", or "nor" at the moment */
-
+/**
+ * Builds and returns a disjunct list for "and", "or" and "nor"
+ * for each disjunct in the label_table, we build three disjuncts
+ * this means that "Danny and Tycho and Billy" will be parsable in
+ * two ways.  I don't know an easy way to avoid this
+ * the string is either "and", or "or", or "nor" at the moment.
+ */
+Disjunct * build_AND_disjunct_list(Sentence sent, char * s)
+{
 	int lab;
 	Disjunct *d_list, *d1, *d3, *d, *d_copy;
 	Connector *c1, *c2, *c3;
@@ -1179,7 +1200,6 @@ Disjunct * build_AND_disjunct_list(Sentence sent, char * s) {
         compares with the image structure for this word.
 */
 
-typedef struct Image_node_struct Image_node;
 struct Image_node_struct {
 	Image_node * next;
 	Connector * c;  /* the connector the place on the disjunct must match */
@@ -1189,75 +1209,90 @@ struct Image_node_struct {
 					   <0 then go -place to the left. */
 };
 
-static Image_node * image_array[MAX_SENTENCE];
-/* points to the image structure for eacch word.  NULL if not a fat word. */
-
-static char has_fat_down[MAX_SENTENCE];  /* TRUE if this word has a fat down link
-									 FALSE otherise */
-int set_has_fat_down(Sentence sent) {
-/* Fill in the has_fat_down array.  Uses link_array[].
-   Returns TRUE if there exists at least one word with a
-   fat down label.
-*/
+/**
+ * Fill in the has_fat_down array.  Uses link_array[].
+ * Returns TRUE if there exists at least one word with a
+ * fat down label.
+ */
+int set_has_fat_down(Sentence sent)
+{
 	int link, w, N_fat;
 	Parse_info pi = sent->parse_info;
 
 	N_fat = 0;
 
-	for (w=0; w<pi->N_words; w++) {
-		has_fat_down[w] = FALSE;
+	for (w = 0; w < pi->N_words; w++)
+	{
+		pi->has_fat_down[w] = FALSE;
 	}
 
-	for (link=0; link<pi->N_links; link++) {
-		if (pi->link_array[link].lc->priority == DOWN_priority) {
+	for (link = 0; link < pi->N_links; link++)
+	{
+		if (pi->link_array[link].lc->priority == DOWN_priority)
+		{
 			N_fat ++;
-			has_fat_down[pi->link_array[link].l] = TRUE;
-		} else if (pi->link_array[link].rc->priority == DOWN_priority) {
+			pi->has_fat_down[pi->link_array[link].l] = TRUE;
+		}
+		else if (pi->link_array[link].rc->priority == DOWN_priority)
+		{
 			N_fat ++;
-			has_fat_down[pi->link_array[link].r] = TRUE;
+			pi->has_fat_down[pi->link_array[link].r] = TRUE;
 		}
 	}
 	return (N_fat > 0);
 }
 
-static void free_image_array(Parse_info pi) {
+static void free_image_array(Parse_info pi)
+{
 	int w;
 	Image_node * in, * inx;
-	for (w=0; w<pi->N_words; w++) {
-		for (in=image_array[w]; in!=NULL; in=inx) {
+	for (w = 0; w < pi->N_words; w++)
+	{
+		for (in = pi->image_array[w]; in != NULL; in = inx)
+		{
 			inx = in->next;
 			xfree((char *)in, sizeof(Image_node));
 		}
+		pi->image_array[w] = NULL;
 	}
 }
 
-static void build_image_array(Sentence sent) {
-/* uses link_array, chosen_disjuncts, and down_label to construct
-   image_array */
+/**
+ * Uses link_array, chosen_disjuncts, and down_label to construct
+ * image_array
+ */
+static void build_image_array(Sentence sent)
+{
 	int link, end, word;
 	Connector * this_end_con, *other_end_con, * upcon, * updiscon, *clist;
 	Disjunct * dis, * updis;
 	Image_node * in;
 	Parse_info pi = sent->parse_info;
 
-	for (word=0; word<pi->N_words; word++) {
-		image_array[word] = NULL;
+	for (word=0; word<pi->N_words; word++)
+	{
+		pi->image_array[word] = NULL;
 	}
 
-	for (end = -1; end <= 1; end += 2) {
-		for (link=0; link<pi->N_links; link++) {
-			if (end<0) {
+	for (end = -1; end <= 1; end += 2)
+	{
+		for (link = 0; link < pi->N_links; link++)
+		{
+			if (end < 0)
+			{
 				word = pi->link_array[link].l;
-				if (!has_fat_down[word]) continue;
+				if (!pi->has_fat_down[word]) continue;
 				this_end_con = pi->link_array[link].lc;
 				other_end_con = pi->link_array[link].rc;
 				dis = pi->chosen_disjuncts[word];
 				clist = dis->right;
-			} else {
-				word =pi->link_array[link].r;
-				if (!has_fat_down[word]) continue;
-				this_end_con =pi->link_array[link].rc;
-				other_end_con =pi->link_array[link].lc;
+			}
+			else
+			{
+				word = pi->link_array[link].r;
+				if (!pi->has_fat_down[word]) continue;
+				this_end_con = pi->link_array[link].rc;
+				other_end_con = pi->link_array[link].lc;
 				dis = pi->chosen_disjuncts[word];
 				clist = dis->left;
 			}
@@ -1269,35 +1304,51 @@ static void build_image_array(Sentence sent) {
 			   or commas links or either/neither links */
 
 			in = (Image_node *) xalloc(sizeof(Image_node));
-			in->next = image_array[word];
-			image_array[word] = in;
+			in->next = pi->image_array[word];
+			pi->image_array[word] = in;
 			in->c = other_end_con;
+
 			/* the rest of this code is for computing in->place */
-			if (this_end_con->priority == UP_priority) {
+			if (this_end_con->priority == UP_priority)
+			{
 				in->place = 0;
-			} else {
+			}
+			else
+			{
 				in->place = 1;
 				if ((dis->left != NULL) &&
-					(dis->left->priority == UP_priority)) {
+					(dis->left->priority == UP_priority))
+				{
 					upcon = dis->left;
-				} else if ((dis->right != NULL) &&
-						   (dis->right->priority == UP_priority)) {
+				}
+				else if ((dis->right != NULL) &&
+						   (dis->right->priority == UP_priority))
+				{
 					upcon = dis->right;
-				} else {
+				}
+				else
+				{
 					upcon = NULL;
 				}
-				if (upcon != NULL) { /* add on extra for a fat up link */
+				if (upcon != NULL)
+				{
+					/* add on extra for a fat up link */
 					updis = sent->and_data.label_table[upcon->label];
-					if (end > 0) {
+					if (end > 0)
+					{
 						updiscon = updis->left;
-					} else {
+					}
+					else
+					{
 						updiscon = updis->right;
 					}
-					for (;updiscon != NULL; updiscon = updiscon->next) {
+					for (;updiscon != NULL; updiscon = updiscon->next)
+					{
 						in->place ++;
 					}
 				}
-				for (; clist != this_end_con; clist = clist->next) {
+				for (; clist != this_end_con; clist = clist->next)
+				{
 					if (clist->label < 0) in->place++;
 				}
 				in->place = in->place * (-end);
@@ -1306,7 +1357,7 @@ static void build_image_array(Sentence sent) {
 	}
 }
 
-/** 
+/**
  * returns TRUE if string s represents a strictly smaller match set
  * than does t
  */
@@ -1326,12 +1377,14 @@ static int strictly_smaller(const char * s, const char * t)
 	return (strictness > 0);
 }
 
-static Disjunct * find_subdisjunct(Sentence sent, Disjunct * dis, int label) {
-/* dis points to a disjunct in the label_table.  label is the label
-   of a different set of disjuncts.  These can be derived from the label
-   of dis.  Find the specific disjunct of in label_table[label]
-   which corresponds to dis.
-*/
+/**
+ * dis points to a disjunct in the label_table.  label is the label
+ * of a different set of disjuncts.  These can be derived from the label
+ * of dis.  Find the specific disjunct of in label_table[label]
+ * which corresponds to dis.
+ */
+static Disjunct * find_subdisjunct(Sentence sent, Disjunct * dis, int label)
+{
 	Disjunct * d;
 	Connector * cx, *cy;
 	for (d=sent->and_data.label_table[label]; d!=NULL; d=d->next) {
@@ -1353,7 +1406,7 @@ static Disjunct * find_subdisjunct(Sentence sent, Disjunct * dis, int label) {
 }
 
 /**
- * is_canonical_linkage -- 
+ * is_canonical_linkage --
  * This uses link_array[], chosen_disjuncts[], has_fat_down[].
  * It assumes that there is a fat link in the current linkage.
  * See the comments above for more information about how it works
@@ -1371,13 +1424,16 @@ int is_canonical_linkage(Sentence sent)
 
 	build_image_array(sent);
 
-	for (w=0; w<pi->N_words; w++) {
-		if (!has_fat_down[w]) continue;
+	for (w=0; w<pi->N_words; w++)
+	{
+		if (!pi->has_fat_down[w]) continue;
 		chosen_d = pi->chosen_disjuncts[w];
 
 		/* there must be a down connector in both the left and right list */
-		for (d_c = chosen_d->left; d_c!=NULL; d_c=d_c->next) {
-			if (d_c->priority == DOWN_priority) {
+		for (d_c = chosen_d->left; d_c!=NULL; d_c=d_c->next)
+		{
+			if (d_c->priority == DOWN_priority)
+			{
 				d_label = d_c->label;
 				break;
 			}
@@ -1396,42 +1452,41 @@ int is_canonical_linkage(Sentence sent)
 
 		/* check that the disjunct on w is minimal (canonical) */
 
-		for (dis=sent->and_data.label_table[d_label]; dis!=NULL; dis=dis->next) {
-
+		for (dis=sent->and_data.label_table[d_label]; dis!=NULL; dis=dis->next)
+		{
 			/* now, reject a disjunct if it's not strictly below the old */
 			if(!strictly_smaller(dis->string, d_c->string)) continue;
 
 			/* Now, it has to match the image connectors */
-
-			for (in=image_array[w]; in!=NULL; in=in->next) {
-
+			for (in = pi->image_array[w]; in != NULL; in = in->next)
+			{
 				place = in->place;
-				if (place == 0) {
-
+				if (place == 0)
+				{
 					assert(upcon != NULL, "Should have found an up link");
 					dummy_connector.label = upcon->label;
 
 					/* now we have to compute the string of the
 					   disjunct with upcon->label that corresponds
 					   to dis  */
-
-					if (upcon->label == d_label) {
+					if (upcon->label == d_label)
+					{
 						dummy_connector.string = dis->string;
 					} else {
 						dummy_connector.string =
 						  find_subdisjunct(sent, dis, upcon->label)->string;
 					}
-					if (!x_match(&dummy_connector, in->c)) break;  /* I hope using x_match here is right */
+					if (!x_match(sent, &dummy_connector, in->c)) break;  /* I hope using x_match here is right */
 				} else if (place > 0) {
 					for (c=dis->right; place > 1; place--) {
 						c = c->next;
 					}
-					if (!x_match(c, in->c)) break;	/* Ditto above comment  --DS 07/97*/
+					if (!x_match(sent, c, in->c)) break;	/* Ditto above comment  --DS 07/97*/
 				} else {
 					for (c=dis->left; place < -1; place++) {
 						c = c->next;
 					}
-					if (!x_match(c, in->c)) break;  /* Ditto Ditto */
+					if (!x_match(sent, c, in->c)) break;  /* Ditto Ditto */
 				}
 			}
 
@@ -1445,34 +1500,39 @@ int is_canonical_linkage(Sentence sent)
 	return (w == pi->N_words);
 }
 
+/**
+ * This takes as input link_array[], sublinkage->link[]->l and
+ * sublinkage->link[]->r (and also has_fat_down[word], which has been
+ * computed in a prior call to is_canonical()), and from these
+ * computes sublinkage->link[].lc and .rc.  We assume these have
+ * been initialized with the values from link_array.  We also assume
+ * that there are fat links.
+ */
 void compute_pp_link_array_connectors(Sentence sent, Sublinkage *sublinkage)
 {
-/*
-   This takes as input link_array[], sublinkage->link[]->l and
-   sublinkage->link[]->r (and also has_fat_down[word], which has been
-   computed in a prior call to is_canonical()), and from these
-   computes sublinkage->link[].lc and .rc.  We assume these have
-   been initialized with the values from link_array.  We also assume
-   that there are fat links.
-*/
 	int link, end, word, place;
 	Connector * this_end_con, * upcon, * updiscon, *clist, *con, *mycon;
 	Disjunct * dis, * updis, *mydis;
 	Parse_info pi = sent->parse_info;
 
-	for (end = -1; end <= 1; end += 2) {
-		for (link=0; link<pi->N_links; link++) {
+	for (end = -1; end <= 1; end += 2)
+	{
+		for (link=0; link<pi->N_links; link++)
+		{
 			if (sublinkage->link[link]->l == -1) continue;
-			if (end<0) {
+			if (end < 0)
+			{
 				word = pi->link_array[link].l;
-				if (!has_fat_down[word]) continue;
+				if (!pi->has_fat_down[word]) continue;
 				this_end_con = pi->link_array[link].lc;
 				dis = pi->chosen_disjuncts[word];
 				mydis = pi->chosen_disjuncts[sublinkage->link[link]->l];
 				clist = dis->right;
-			} else {
+			}
+			else
+			{
 				word = pi->link_array[link].r;
-				if (!has_fat_down[word]) continue;
+				if (!pi->has_fat_down[word]) continue;
 				this_end_con = pi->link_array[link].rc;
 				dis = pi->chosen_disjuncts[word];
 				mydis = pi->chosen_disjuncts[sublinkage->link[link]->r];
