@@ -15,22 +15,30 @@
 
 /* This file contains the exhaustive search algorithm. */
 
-static char ** deletable;
-static char ** effective_dist; 
-static Word *  local_sent;
-static int	 null_block, islands_ok, null_links;
-static Resources current_resources;
+struct count_context_s
+{
+	char ** deletable;
+	char ** effective_dist; 
+	Word *  local_sent;
+	int     null_block;
+	int     islands_ok;
+	int     null_links;
+	Resources current_resources;
+};
 
-int x_match(Connector *a, Connector *b) {
-	return match(a, b, 0, 0);
+int x_match(Sentence sent, Connector *a, Connector *b)
+{
+	return match(sent, a, b, 0, 0);
 }
 
-void count_set_effective_distance(Sentence sent) {
-	effective_dist = sent->effective_dist;
+void count_set_effective_distance(Sentence sent)
+{
+	sent->count_ctxt->effective_dist = sent->effective_dist;
 }
 
-void count_unset_effective_distance(Sentence sent) {
-	effective_dist = NULL;
+void count_unset_effective_distance(Sentence sent)
+{
+	sent->count_ctxt->effective_dist = NULL;
 }
 
 /*
@@ -46,18 +54,24 @@ void count_unset_effective_distance(Sentence sent) {
  * is different depending on which of the two priority cases is being
  * considered.  See the comments below. 
  */
-int match(Connector *a, Connector *b, int aw, int bw)
+int match(Sentence sent, Connector *a, Connector *b, int aw, int bw)
 {
 	const char *s, *t;
 	int x, y, dist;
+	count_context_t *ctxt;
+
 	if (a->label != b->label) return FALSE;
+
+	ctxt = sent->count_ctxt;
+
 	x = a->priority;
 	y = b->priority;
 
 	s = a->string;
 	t = b->string;
 
-	while(isupper((int)*s) || isupper((int)*t)) {
+	while(isupper((int)*s) || isupper((int)*t))
+	{
 		if (*s != *t) return FALSE;
 		s++;
 		t++;
@@ -67,7 +81,7 @@ int match(Connector *a, Connector *b, int aw, int bw)
 		dist = 0;
 	} else {
 		assert(aw < bw, "match() did not receive params in the natural order.");
-		dist = effective_dist[aw][bw];
+		dist = ctxt->effective_dist[aw][bw];
 	}
 	/*	printf("M: a=%4s b=%4s  ap=%d bp=%d  aw=%d  bw=%d  a->ll=%d b->ll=%d  dist=%d\n",
 		   s, t, x, y, aw, bw, a->length_limit, b->length_limit, dist); */
@@ -163,7 +177,8 @@ static int hash(int lw, int rw, Connector *le, Connector *re, int cost) {
 	return i & (table_size-1);
 }
 
-void free_table(Sentence sent) {
+void free_table(Sentence sent)
+{
 	int i;
 	Table_connector *t, *x;
 
@@ -197,9 +212,11 @@ static Table_connector * table_store(int lw, int rw,
 }
 
 /** returns the pointer to this info, NULL if not there */
-static Table_connector * find_table_pointer(int lw, int rw, 
-                                       Connector *le, Connector *re,
-                                       int cost)
+static Table_connector * 
+find_table_pointer(count_context_t *ctxt,
+                   int lw, int rw, 
+                   Connector *le, Connector *re,
+                   int cost)
 {
 	Table_connector *t;
 	t = table[hash(lw, rw, le, re, cost)];
@@ -211,16 +228,19 @@ static Table_connector * find_table_pointer(int lw, int rw,
 	/* Create a new connector only if resources are exhausted.
 	 * (???) Huh? I guess we're in panic parse mode in that case.
 	 */
-	if ((current_resources != NULL) && resources_exhausted(current_resources)) {
+	if ((ctxt->current_resources != NULL) && 
+	     resources_exhausted(ctxt->current_resources))
+	{
 		return table_store(lw, rw, le, re, cost, 0);
 	}
 	else return NULL;
 }
 
 /** returns the count for this quintuple if there, -1 otherwise */
-s64 table_lookup(int lw, int rw, Connector *le, Connector *re, int cost)
+s64 table_lookup(Sentence sent, 
+                 int lw, int rw, Connector *le, Connector *re, int cost)
 {
-	Table_connector *t = find_table_pointer(lw, rw, le, re, cost);
+	Table_connector *t = find_table_pointer(sent->count_ctxt, lw, rw, le, re, cost);
 
 	if (t == NULL) return -1; else return t->count;
 }
@@ -229,11 +249,11 @@ s64 table_lookup(int lw, int rw, Connector *le, Connector *re, int cost)
  * Stores the value in the table.  Unlike table_store, it assumes 
  * it's already there
  */
-static void table_update(int lw, int rw, 
+static void table_update(count_context_t *ctxt, int lw, int rw, 
                          Connector *le, Connector *re,
                          int cost, s64 count)
 {
-	Table_connector *t = find_table_pointer(lw, rw, le, re, cost);
+	Table_connector *t = find_table_pointer(ctxt, lw, rw, le, re, cost);
 
 	assert(t != NULL, "This entry is supposed to be in the table.");
 	t->count = count;
@@ -243,14 +263,16 @@ static void table_update(int lw, int rw,
  * Returns 0 if and only if this entry is in the hash table 
  * with a count value of 0.
  */
-static s64 pseudocount(int lw, int rw, Connector *le, Connector *re, int cost)
+static s64 pseudocount(Sentence sent,
+                       int lw, int rw, Connector *le, Connector *re, int cost)
 {
 	s64 count;
-	count = table_lookup(lw, rw, le, re, cost);
+	count = table_lookup(sent, lw, rw, le, re, cost);
 	if (count == 0) return 0; else return 1;
 }
 
-static s64 count(int lw, int rw, Connector *le, Connector *re, int cost)
+static s64 count(Sentence sent, int lw, int rw,
+                 Connector *le, Connector *re, int cost)
 {
 	Disjunct * d;
 	s64 total, pseudototal;
@@ -261,9 +283,11 @@ static s64 count(int lw, int rw, Connector *le, Connector *re, int cost)
 	Match_node * m, *m1;
 	Table_connector *t;
 
+	count_context_t *ctxt = sent->count_ctxt;
+
 	if (cost < 0) return 0;  /* will we ever call it with cost<0 ? */
 
-	t = find_table_pointer(lw, rw, le, re, cost);
+	t = find_table_pointer(sent->count_ctxt, lw, rw, le, re, cost);
 
 	if (t == NULL) {
 		/* Create the table entry with a tentative cost of 0. 
@@ -285,11 +309,11 @@ static s64 count(int lw, int rw, Connector *le, Connector *re, int cost)
 	}
 
 	if ((le == NULL) && (re == NULL)) {
-		if (!islands_ok && (lw != -1)) {
+		if (!ctxt->islands_ok && (lw != -1)) {
 		  /* if we don't allow islands (a set of words linked together but
 			 separate from the rest of the sentence) then  the cost of skipping
 			 n words is just n */
-			if (cost == ((rw-lw-1)+null_block-1)/null_block) {
+			if (cost == ((rw-lw-1) + ctxt->null_block-1)/ctxt->null_block) {
 				/* if null_block=4 then the cost of
 				   1,2,3,4 nulls is 1, 5,6,7,8 is 2 etc. */
 				t->count = 1;
@@ -306,12 +330,12 @@ static s64 count(int lw, int rw, Connector *le, Connector *re, int cost)
 		} else {
 			total = 0;
 			w = lw+1;
-			for (d = local_sent[w].d; d != NULL; d = d->next) {
+			for (d = ctxt->local_sent[w].d; d != NULL; d = d->next) {
 				if (d->left == NULL) {
-					total += count(w, rw, d->right, NULL, cost-1);
+					total += count(sent, w, rw, d->right, NULL, cost-1);
 				}
 			}
-			total += count(w, rw, NULL, NULL, cost-1);
+			total += count(sent, w, rw, NULL, NULL, cost-1);
 			t->count = total;
 		}
 		return t->count;
@@ -332,7 +356,7 @@ static s64 count(int lw, int rw, Connector *le, Connector *re, int cost)
 	total = 0;
 
 	for (w=start_word; w < end_word+1; w++) {
-		m1 = m = form_match_list(w, le, lw, re, rw);
+		m1 = m = form_match_list(sent, w, le, lw, re, rw);
 		for (; m!=NULL; m=m->next) {
 			d = m->d;
 			for (lcost = 0; lcost <= cost; lcost++) {
@@ -342,67 +366,68 @@ static s64 count(int lw, int rw, Connector *le, Connector *re, int cost)
 
 				/* Now, we determine if (based on table only) we can see that
 				   the current range is not parsable. */
-				Lmatch = (le != NULL) && (d->left != NULL) && match(le, d->left, lw, w);
-				Rmatch = (d->right != NULL) && (re != NULL) && match(d->right, re, w, rw);
+				Lmatch = (le != NULL) && (d->left != NULL) && match(sent, le, d->left, lw, w);
+				Rmatch = (d->right != NULL) && (re != NULL) && match(sent, d->right, re, w, rw);
 
 				rightcount = leftcount = 0;
 				if (Lmatch) {
-					leftcount = pseudocount(lw, w, le->next, d->left->next, lcost);
-					if (le->multi) leftcount += pseudocount(lw, w, le, d->left->next, lcost);
-					if (d->left->multi) leftcount += pseudocount(lw, w, le->next, d->left, lcost);
-					if (le->multi && d->left->multi) leftcount += pseudocount(lw, w, le, d->left, lcost);
+					leftcount = pseudocount(sent, lw, w, le->next, d->left->next, lcost);
+					if (le->multi) leftcount += pseudocount(sent, lw, w, le, d->left->next, lcost);
+					if (d->left->multi) leftcount += pseudocount(sent, lw, w, le->next, d->left, lcost);
+					if (le->multi && d->left->multi) leftcount += pseudocount(sent, lw, w, le, d->left, lcost);
 				}
 
 				if (Rmatch) {
-					rightcount = pseudocount(w, rw, d->right->next, re->next, rcost);
-					if (d->right->multi) rightcount += pseudocount(w,rw,d->right,re->next, rcost);
-					if (re->multi) rightcount += pseudocount(w, rw, d->right->next, re, rcost);
-					if (d->right->multi && re->multi) rightcount += pseudocount(w, rw, d->right, re, rcost);
+					rightcount = pseudocount(sent, w, rw, d->right->next, re->next, rcost);
+					if (d->right->multi) rightcount += pseudocount(sent, w,rw,d->right,re->next, rcost);
+					if (re->multi) rightcount += pseudocount(sent, w, rw, d->right->next, re, rcost);
+					if (d->right->multi && re->multi) rightcount += pseudocount(sent, w, rw, d->right, re, rcost);
 				}
 
-				pseudototal = leftcount*rightcount;  /* total number where links are used on both sides */
+				/* total number where links are used on both sides */
+				pseudototal = leftcount*rightcount;
 
 				if (leftcount > 0) {
 					/* evaluate using the left match, but not the right */
-					pseudototal += leftcount * pseudocount(w, rw, d->right, re, rcost);
+					pseudototal += leftcount * pseudocount(sent, w, rw, d->right, re, rcost);
 				}
 				if ((le == NULL) && (rightcount > 0)) {
 					/* evaluate using the right match, but not the left */
-					pseudototal += rightcount * pseudocount(lw, w, le, d->left, lcost);
+					pseudototal += rightcount * pseudocount(sent, lw, w, le, d->left, lcost);
 				}
 
 				/* now pseudototal is 0 implies that we know that the true total is 0 */
 				if (pseudototal != 0) {
 					rightcount = leftcount = 0;
 					if (Lmatch) {
-						leftcount = count(lw, w, le->next, d->left->next, lcost);
-						if (le->multi) leftcount += count(lw, w, le, d->left->next, lcost);
-						if (d->left->multi) leftcount += count(lw, w, le->next, d->left, lcost);
-						if (le->multi && d->left->multi) leftcount += count(lw, w, le, d->left, lcost);
+						leftcount = count(sent, lw, w, le->next, d->left->next, lcost);
+						if (le->multi) leftcount += count(sent, lw, w, le, d->left->next, lcost);
+						if (d->left->multi) leftcount += count(sent, lw, w, le->next, d->left, lcost);
+						if (le->multi && d->left->multi) leftcount += count(sent, lw, w, le, d->left, lcost);
 					}
 
 					if (Rmatch) {
-						rightcount = count(w, rw, d->right->next, re->next, rcost);
-						if (d->right->multi) rightcount += count(w,rw,d->right,re->next, rcost);
-						if (re->multi) rightcount += count(w, rw, d->right->next, re, rcost);
-						if (d->right->multi && re->multi) rightcount += count(w, rw, d->right, re, rcost);
+						rightcount = count(sent, w, rw, d->right->next, re->next, rcost);
+						if (d->right->multi) rightcount += count(sent, w,rw,d->right,re->next, rcost);
+						if (re->multi) rightcount += count(sent, w, rw, d->right->next, re, rcost);
+						if (d->right->multi && re->multi) rightcount += count(sent, w, rw, d->right, re, rcost);
 					}
 
 					total += leftcount*rightcount;  /* total number where links are used on both sides */
 
 					if (leftcount > 0) {
 						/* evaluate using the left match, but not the right */
-						total += leftcount * count(w, rw, d->right, re, rcost);
+						total += leftcount * count(sent, w, rw, d->right, re, rcost);
 					}
 					if ((le == NULL) && (rightcount > 0)) {
 						/* evaluate using the right match, but not the left */
-						total += rightcount * count(lw, w, le, d->left, lcost);
+						total += rightcount * count(sent, lw, w, le, d->left, lcost);
 					}
 				}
 			}
 		}
 
-		put_match_list(m1);
+		put_match_list(sent, m1);
 	}
 	t->count = total;
 	return total;
@@ -416,15 +441,16 @@ static s64 count(int lw, int rw, Connector *le, Connector *re, int cost)
 s64 parse(Sentence sent, int cost, Parse_Options opts)
 {
 	s64 total;
+	count_context_t *ctxt = sent->count_ctxt;
 
 	count_set_effective_distance(sent);
-	current_resources = opts->resources;
-	local_sent = sent->word;
-	deletable = sent->deletable;
-	null_block = opts->null_block;
-	islands_ok = opts->islands_ok;
+	ctxt->current_resources = opts->resources;
+	ctxt->local_sent = sent->word;
+	ctxt->deletable = sent->deletable;
+	ctxt->null_block = opts->null_block;
+	ctxt->islands_ok = opts->islands_ok;
 
-	total = count(-1, sent->length, NULL, NULL, cost+1);
+	total = count(sent, -1, sent->length, NULL, NULL, cost+1);
 	if (verbosity > 1) {
 		printf("Total count with %d null links:   %lld\n", cost, total);
 	}
@@ -432,8 +458,8 @@ s64 parse(Sentence sent, int cost, Parse_Options opts)
 		printf("WARNING: Overflow in count! cnt=%lld\n", total);
 	}
 
-	local_sent = NULL;
-	current_resources = NULL;
+	ctxt->local_sent = NULL;
+	ctxt->current_resources = NULL;
 	return total;
 }
 
@@ -469,12 +495,13 @@ s64 parse(Sentence sent, int cost, Parse_Options opts)
    2  This region can be completed, and it's been marked.
    */
 
-static int x_prune_match(Connector *le, Connector *re, int lw, int rw)
+static int x_prune_match(count_context_t *ctxt,
+                         Connector *le, Connector *re, int lw, int rw)
 {
 	int dist;
 
 	assert(lw < rw, "prune_match() did not receive params in the natural order.");
-	dist = effective_dist[lw][rw];
+	dist = ctxt->effective_dist[lw][rw];
 	return prune_match(dist, le, re);
 }
 
@@ -483,7 +510,7 @@ static int x_prune_match(Connector *le, Connector *re, int lw, int rw)
  * links.  Returns 1 if it can, and it's not been marked, and returns
  * 2 if it can and it has been marked.
  */
-static int region_valid(int lw, int rw, Connector *le, Connector *re)
+static int region_valid(Sentence sent, int lw, int rw, Connector *le, Connector *re)
 {
 	Disjunct * d;
 	int left_valid, right_valid, found;
@@ -491,10 +518,12 @@ static int region_valid(int lw, int rw, Connector *le, Connector *re)
 	int w;
 	Match_node * m, *m1;
 
-	i = table_lookup(lw, rw, le, re, 0);
+	count_context_t *ctxt = sent->count_ctxt;
+
+	i = table_lookup(sent, lw, rw, le, re, 0);
 	if (i >= 0) return i;
 
-	if ((le == NULL) && (re == NULL) && deletable[lw][rw]) {
+	if ((le == NULL) && (re == NULL) && ctxt->deletable[lw][rw]) {
 		table_store(lw, rw, le, re, 0, 1);
 		return 1;
 	}
@@ -513,32 +542,32 @@ static int region_valid(int lw, int rw, Connector *le, Connector *re)
 	found = 0;
 
 	for (w=start_word; w < end_word+1; w++) {
-		m1 = m = form_match_list(w, le, lw, re, rw);
+		m1 = m = form_match_list(sent, w, le, lw, re, rw);
 		for (; m!=NULL; m=m->next) {
 			d = m->d;
 			/* mark_cost++;*/
 			/* in the following expressions we use the fact that 0=FALSE. Could eliminate
 			   by always saying "region_valid(...) != 0"  */
-			left_valid = (((le != NULL) && (d->left != NULL) && x_prune_match(le, d->left, lw, w)) &&
-						  ((region_valid(lw, w, le->next, d->left->next)) ||
-						   ((le->multi) && region_valid(lw, w, le, d->left->next)) ||
-						   ((d->left->multi) && region_valid(lw, w, le->next, d->left)) ||
-						   ((le->multi && d->left->multi) && region_valid(lw, w, le, d->left))));
-			if (left_valid && region_valid(w, rw, d->right, re)) {
+			left_valid = (((le != NULL) && (d->left != NULL) && x_prune_match(ctxt, le, d->left, lw, w)) &&
+						  ((region_valid(sent, lw, w, le->next, d->left->next)) ||
+						   ((le->multi) && region_valid(sent, lw, w, le, d->left->next)) ||
+						   ((d->left->multi) && region_valid(sent, lw, w, le->next, d->left)) ||
+						   ((le->multi && d->left->multi) && region_valid(sent, lw, w, le, d->left))));
+			if (left_valid && region_valid(sent, w, rw, d->right, re)) {
 				found = 1;
 				break;
 			}
-			right_valid = (((d->right != NULL) && (re != NULL) && x_prune_match(d->right, re, w, rw)) &&
-						   ((region_valid(w, rw, d->right->next,re->next))	||
-							((d->right->multi) && region_valid(w,rw,d->right,re->next))  ||
-							((re->multi) && region_valid(w, rw, d->right->next, re))  ||
-							((d->right->multi && re->multi) && region_valid(w, rw, d->right, re))));
-			if ((left_valid && right_valid) || (right_valid && region_valid(lw, w, le, d->left))) {
+			right_valid = (((d->right != NULL) && (re != NULL) && x_prune_match(ctxt, d->right, re, w, rw)) &&
+						   ((region_valid(sent, w, rw, d->right->next,re->next))	||
+							((d->right->multi) && region_valid(sent, w,rw,d->right,re->next))  ||
+							((re->multi) && region_valid(sent, w, rw, d->right->next, re))  ||
+							((d->right->multi && re->multi) && region_valid(sent, w, rw, d->right, re))));
+			if ((left_valid && right_valid) || (right_valid && region_valid(sent, lw, w, le, d->left))) {
 				found = 1;
 				break;
 			}
 		}
-		put_match_list(m1);
+		put_match_list(sent, m1);
 		if (found != 0) break;
 	}
 	table_store(lw, rw, le, re, 0, found);
@@ -552,7 +581,8 @@ static int region_valid(int lw, int rw, Connector *le, Connector *re)
  * this region itself is not valid, then this fact will be recorded
  * in the table, and nothing else happens.
  */
-static void mark_region(int lw, int rw, Connector *le, Connector *re)
+static void mark_region(Sentence sent,
+                        int lw, int rw, Connector *le, Connector *re)
 {
 
 	Disjunct * d;
@@ -560,21 +590,22 @@ static void mark_region(int lw, int rw, Connector *le, Connector *re)
 	int start_word, end_word;
 	int w;
 	Match_node * m, *m1;
+	count_context_t *ctxt = sent->count_ctxt;
 
-	i = region_valid(lw, rw, le, re);
+	i = region_valid(sent, lw, rw, le, re);
 	if ((i==0) || (i==2)) return;
 	/* we only reach this point if it's a valid unmarked region, i=1 */
-	table_update(lw, rw, le, re, 0, 2);
+	table_update(ctxt, lw, rw, le, re, 0, 2);
 
-	if ((le == NULL) && (re == NULL) && (null_links) && (rw != 1+lw)) {
+	if ((le == NULL) && (re == NULL) && (ctxt->null_links) && (rw != 1+lw)) {
 		w = lw+1;
-		for (d = local_sent[w].d; d != NULL; d = d->next) {
-			if ((d->left == NULL) && region_valid(w, rw, d->right, NULL)) {
+		for (d = ctxt->local_sent[w].d; d != NULL; d = d->next) {
+			if ((d->left == NULL) && region_valid(sent, w, rw, d->right, NULL)) {
 				d->marked = TRUE;
-				mark_region(w, rw, d->right, NULL);
+				mark_region(sent, w, rw, d->right, NULL);
 			}
 		}
-		mark_region(w, rw, NULL, NULL);
+		mark_region(sent, w, rw, NULL, NULL);
 		return;
 	}
 
@@ -590,60 +621,61 @@ static void mark_region(int lw, int rw, Connector *le, Connector *re)
 	}
 
 	for (w=start_word; w < end_word+1; w++) {
-		m1 = m = form_match_list(w, le, lw, re, rw);
+		m1 = m = form_match_list(sent, w, le, lw, re, rw);
 		for (; m!=NULL; m=m->next) {
 			d = m->d;
 			/* mark_cost++;*/
-			left_valid = (((le != NULL) && (d->left != NULL) && x_prune_match(le, d->left, lw, w)) &&
-						  ((region_valid(lw, w, le->next, d->left->next)) ||
-						   ((le->multi) && region_valid(lw, w, le, d->left->next)) ||
-						   ((d->left->multi) && region_valid(lw, w, le->next, d->left)) ||
-						   ((le->multi && d->left->multi) && region_valid(lw, w, le, d->left))));
-			right_valid = (((d->right != NULL) && (re != NULL) && x_prune_match(d->right, re, w, rw)) &&
-						   ((region_valid(w, rw, d->right->next,re->next)) ||
-							((d->right->multi) && region_valid(w,rw,d->right,re->next))  ||
-							((re->multi) && region_valid(w, rw, d->right->next, re)) ||
-							((d->right->multi && re->multi) && region_valid(w, rw, d->right, re))));
+			left_valid = (((le != NULL) && (d->left != NULL) && x_prune_match(ctxt, le, d->left, lw, w)) &&
+						  ((region_valid(sent, lw, w, le->next, d->left->next)) ||
+						   ((le->multi) && region_valid(sent, lw, w, le, d->left->next)) ||
+						   ((d->left->multi) && region_valid(sent, lw, w, le->next, d->left)) ||
+						   ((le->multi && d->left->multi) && region_valid(sent, lw, w, le, d->left))));
+			right_valid = (((d->right != NULL) && (re != NULL) && x_prune_match(ctxt, d->right, re, w, rw)) &&
+						   ((region_valid(sent, w, rw, d->right->next,re->next)) ||
+							((d->right->multi) && region_valid(sent, w,rw,d->right,re->next))  ||
+							((re->multi) && region_valid(sent, w, rw, d->right->next, re)) ||
+							((d->right->multi && re->multi) && region_valid(sent, w, rw, d->right, re))));
 
 			/* The following if statements could be restructured to avoid superfluous calls
 			   to mark_region.  It didn't seem a high priority, so I didn't optimize this.
 			   */
 
-			if (left_valid && region_valid(w, rw, d->right, re)) {
+			if (left_valid && region_valid(sent, w, rw, d->right, re)) {
 				d->marked = TRUE;
-				mark_region(w, rw, d->right, re);
-				mark_region(lw, w, le->next, d->left->next);
-				if (le->multi) mark_region(lw, w, le, d->left->next);
-				if (d->left->multi) mark_region(lw, w, le->next, d->left);
-				if (le->multi && d->left->multi) mark_region(lw, w, le, d->left);
+				mark_region(sent, w, rw, d->right, re);
+				mark_region(sent, lw, w, le->next, d->left->next);
+				if (le->multi) mark_region(sent, lw, w, le, d->left->next);
+				if (d->left->multi) mark_region(sent, lw, w, le->next, d->left);
+				if (le->multi && d->left->multi) mark_region(sent, lw, w, le, d->left);
 			}
 
-			if (right_valid && region_valid(lw, w, le, d->left)) {
+			if (right_valid && region_valid(sent, lw, w, le, d->left)) {
 				d->marked = TRUE;
-				mark_region(lw, w, le, d->left);
-				mark_region(w, rw, d->right->next,re->next);
-				if (d->right->multi) mark_region(w,rw,d->right,re->next);
-				if (re->multi) mark_region(w, rw, d->right->next, re);
-				if (d->right->multi && re->multi) mark_region(w, rw, d->right, re);
+				mark_region(sent, lw, w, le, d->left);
+				mark_region(sent, w, rw, d->right->next,re->next);
+				if (d->right->multi) mark_region(sent, w,rw,d->right,re->next);
+				if (re->multi) mark_region(sent, w, rw, d->right->next, re);
+				if (d->right->multi && re->multi) mark_region(sent, w, rw, d->right, re);
 			}
 
 			if (left_valid && right_valid) {
 				d->marked = TRUE;
-				mark_region(lw, w, le->next, d->left->next);
-				if (le->multi) mark_region(lw, w, le, d->left->next);
-				if (d->left->multi) mark_region(lw, w, le->next, d->left);
-				if (le->multi && d->left->multi) mark_region(lw, w, le, d->left);
-				mark_region(w, rw, d->right->next,re->next);
-				if (d->right->multi) mark_region(w,rw,d->right,re->next);
-				if (re->multi) mark_region(w, rw, d->right->next, re);
-				if (d->right->multi && re->multi) mark_region(w, rw, d->right, re);
+				mark_region(sent, lw, w, le->next, d->left->next);
+				if (le->multi) mark_region(sent, lw, w, le, d->left->next);
+				if (d->left->multi) mark_region(sent, lw, w, le->next, d->left);
+				if (le->multi && d->left->multi) mark_region(sent, lw, w, le, d->left);
+				mark_region(sent, w, rw, d->right->next,re->next);
+				if (d->right->multi) mark_region(sent, w,rw,d->right,re->next);
+				if (re->multi) mark_region(sent, w, rw, d->right->next, re);
+				if (d->right->multi && re->multi) mark_region(sent, w, rw, d->right, re);
 			}
 		}
-		put_match_list(m1);
+		put_match_list(sent, m1);
 	}
 }
 
-void delete_unmarked_disjuncts(Sentence sent){
+void delete_unmarked_disjuncts(Sentence sent)
+{
 	int w;
 	Disjunct *d_head, *d, *dx;
 
@@ -677,9 +709,10 @@ void conjunction_prune(Sentence sent, Parse_Options opts)
 {
 	Disjunct * d;
 	int w;
+	count_context_t *ctxt = sent->count_ctxt;
 
-	current_resources = opts->resources;
-	deletable = sent->deletable;
+	ctxt->current_resources = opts->resources;
+	ctxt->deletable = sent->deletable;
 	count_set_effective_distance(sent);
 
 	/* We begin by unmarking all disjuncts.  This would not be necessary if
@@ -695,28 +728,28 @@ void conjunction_prune(Sentence sent, Parse_Options opts)
 
 	init_fast_matcher(sent);
 	init_table(sent);
-	local_sent = sent->word;
-	null_links = (opts->min_null_count > 0);
+	ctxt->local_sent = sent->word;
+	ctxt->null_links = (opts->min_null_count > 0);
 	/*
 	for (d = sent->word[0].d; d != NULL; d = d->next) {
-		if ((d->left == NULL) && region_valid(0, sent->length, d->right, NULL)) {
-			mark_region(0, sent->length, d->right, NULL);
+		if ((d->left == NULL) && region_valid(sent, 0, sent->length, d->right, NULL)) {
+			mark_region(sent, 0, sent->length, d->right, NULL);
 			d->marked = TRUE;
 		}
 	}
-	mark_region(0, sent->length, NULL, NULL);
+	mark_region(sent, 0, sent->length, NULL, NULL);
 	*/
 
-	if (null_links) {
-		mark_region(-1, sent->length, NULL, NULL);
+	if (ctxt->null_links) {
+		mark_region(sent, -1, sent->length, NULL, NULL);
 	} else {
 		for (w=0; w<sent->length; w++) {
 		  /* consider removing the words [0,w-1] from the beginning
 			 of the sentence */
-			if (deletable[-1][w]) {
+			if (ctxt->deletable[-1][w]) {
 				for (d = sent->word[w].d; d != NULL; d = d->next) {
-					if ((d->left == NULL) && region_valid(w, sent->length, d->right, NULL)) {
-						mark_region(w, sent->length, d->right, NULL);
+					if ((d->left == NULL) && region_valid(sent, w, sent->length, d->right, NULL)) {
+						mark_region(sent, w, sent->length, d->right, NULL);
 						d->marked = TRUE;
 					}
 				}
@@ -729,8 +762,20 @@ void conjunction_prune(Sentence sent, Parse_Options opts)
 	free_fast_matcher(sent);
 	free_table(sent);
 
-	local_sent = NULL;
-	current_resources = NULL;
-	deletable = NULL;
+	ctxt->local_sent = NULL;
+	ctxt->current_resources = NULL;
+	ctxt->deletable = NULL;
 	count_unset_effective_distance(sent);
 }
+
+void init_count(Sentence sent)
+{
+	sent->count_ctxt = (count_context_t *) malloc (sizeof(count_context_t));
+	bzero(sent->count_ctxt, sizeof(count_context_t));
+}
+
+void free_count(Sentence sent)
+{
+	if (sent->count_ctxt != NULL) free(sent->count_ctxt);
+	sent->count_ctxt = NULL;
+}
