@@ -13,16 +13,7 @@
 #ifndef _ERRORH_
 #define _ERRORH_
 
-typedef enum {
-    NODICT=1,
-    DICTPARSE,
-    WORDFILE,
-    SEPARATE,
-    NOTINDICT,
-    CHARSET,
-    BUILDEXPR,
-    INTERNALERROR,
-}   LP_error_type;
+#include "link-includes.h"
 
 #if     __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ > 4)
 #define GNUC_PRINTF( format_idx, arg_idx )    \
@@ -31,8 +22,8 @@ typedef enum {
 #define GNUC_PRINTF( format_idx, arg_idx )
 #endif
 
-void lperror(int lperr, const char *fmt, ...) GNUC_PRINTF(2,3);
-void error(const char *fmt, ...) GNUC_PRINTF(1,2);
+void error_report_set_sentence(const Sentence s);
+void prt_error(const char *fmt, ...) GNUC_PRINTF(1,2);
 
 #endif
 
