@@ -13,12 +13,8 @@
 
 #include <link-grammar/api.h>
 
-static int null_links;
-
-static void print_statistics(void) {
-}
-
-void free_deletable(Sentence sent) {
+void free_deletable(Sentence sent)
+{
 	int w;
 	if (sent->deletable != NULL) {
 		for (w = -1; w<sent->length; w++) {
@@ -30,7 +26,8 @@ void free_deletable(Sentence sent) {
 	}
 }
 
-static void build_deletable(Sentence sent, int has_conjunction) {
+static void build_deletable(Sentence sent, int has_conjunction)
+{
 /* Initialize the array deletable[i][j] to indicate if the words		   */
 /* i+1...j-1 could be non existant in one of the multiple linkages.  This  */
 /* array is used in conjunction_prune and power_prune.  Regions of length  */
@@ -58,7 +55,7 @@ static void build_deletable(Sentence sent, int has_conjunction) {
 		for (j=0; j<= sent->length; j++) {
 			if (j == i+1) {
 				sent->deletable[i][j] = TRUE;
-			} else if (null_links) {
+			} else if (sent->null_links) {
 				sent->deletable[i][j] = TRUE;
 			} else if (!has_conjunction) {
 				sent->deletable[i][j] = FALSE;
@@ -87,7 +84,8 @@ static void build_deletable(Sentence sent, int has_conjunction) {
 	}
 }
 
-void free_effective_dist(Sentence sent) {
+void free_effective_dist(Sentence sent)
+{
 	int w;
 	if (sent->effective_dist != NULL) {
 		for (w=0; w<sent->length; w++) {
@@ -98,27 +96,27 @@ void free_effective_dist(Sentence sent) {
 	}
 }
 
-static void build_effective_dist(Sentence sent, int has_conjunction) {
-  /*
-	The "effective distance" between two words is the actual distance minus
-	the largest deletable region strictly between the two words.  If the
-	effective distance between two words is greater than a connector's max
-	link length, then that connector cannot be satisfied by linking these
-	two words.
-
-	[Note: The effective distance is not monotically increasing as you move
-	away from a word.]
-
-	This function creates effective_dist[][].  It assumes that deleteble[][]
-	has already been computed.
-
-	Dynamic programming is used to compute this.  The order used is smallest
-	region to largest.
-
-	Just as deletable[i][j] is constructed for j=N_words (which is one
-	off the end of the sentence) we do that for effective_dist[][].
-  */
-
+/**
+ * The "effective distance" between two words is the actual distance minus
+ * the largest deletable region strictly between the two words.  If the
+ * effective distance between two words is greater than a connector's max
+ * link length, then that connector cannot be satisfied by linking these
+ * two words.
+ *
+ * [Note: The effective distance is not monotically increasing as you move
+ * away from a word.]
+ *
+ * This function creates effective_dist[][].  It assumes that deleteble[][]
+ * has already been computed.
+ *
+ * Dynamic programming is used to compute this.  The order used is smallest
+ * region to largest.
+ *
+ * Just as deletable[i][j] is constructed for j=N_words (which is one
+ * off the end of the sentence) we do that for effective_dist[][].
+ */
+static void build_effective_dist(Sentence sent, int has_conjunction)
+{
 	int i, j, diff;
 
 	free_effective_dist(sent);
@@ -136,7 +134,7 @@ static void build_effective_dist(Sentence sent, int has_conjunction) {
 
 	/* what is the rationale for ignoring the effective_dist
 	   if null links are allowed? */
-	if (null_links) {
+	if (sent->null_links) {
 		for (i=0; i<sent->length; i++) {
 			for (j=0; j<=sent->length; j++) {
 				sent->effective_dist[i][j] = j-i;
@@ -181,9 +179,12 @@ static void build_effective_dist(Sentence sent, int has_conjunction) {
 	*/
 }
 
-void install_fat_connectors(Sentence sent) {
-/* Installs all the special fat disjuncts on all of the words of the   */
-/* sentence */
+/**
+ * Installs all the special fat disjuncts on all of the words of the
+ * sentence
+ */
+void install_fat_connectors(Sentence sent)
+{
 	int i;
 	for (i=0; i<sent->length; i++) {
 		if (sent->is_conjunction[i]) {
@@ -201,7 +202,8 @@ void install_fat_connectors(Sentence sent) {
 }
 
 static void 
-set_connector_list_length_limit(Connector *c,
+set_connector_list_length_limit(Sentence sent,
+                                Connector *c,
                                 Connector_set *conset,
                                 int short_len,
                                 Parse_Options opts)
@@ -210,7 +212,7 @@ set_connector_list_length_limit(Connector *c,
 		if (parse_options_get_all_short_connectors(opts)) {
 			c->length_limit = short_len;
 		}
-		else if (conset == NULL || match_in_connector_set(conset, c, '+')) {
+		else if (conset == NULL || match_in_connector_set(sent, conset, c, '+')) {
 			c->length_limit = UNLIMITED_LEN;
 		} else {
 			c->length_limit = short_len;
@@ -229,8 +231,8 @@ static void set_connector_length_limits(Sentence sent, Parse_Options opts)
 
 	for (i=0; i<sent->length; i++) {
 		for (d = sent->word[i].d; d != NULL; d = d->next) {
-			set_connector_list_length_limit(d->left, sent->dict->unlimited_connector_set, len, opts);
-			set_connector_list_length_limit(d->right, sent->dict->unlimited_connector_set, len, opts);
+			set_connector_list_length_limit(sent, d->left, sent->dict->unlimited_connector_set, len, opts);
+			set_connector_list_length_limit(sent, d->right, sent->dict->unlimited_connector_set, len, opts);
 		}
 	}
 }
@@ -243,7 +245,8 @@ void free_sentence_expressions(Sentence sent)
   }
 }
 
-void free_sentence_disjuncts(Sentence sent) {
+void free_sentence_disjuncts(Sentence sent)
+{
 	int i;
 
 	for (i=0; i<sent->length; ++i) {
@@ -279,7 +282,7 @@ void prepare_to_parse(Sentence sent, Parse_Options opts)
 		print_disjunct_counts(sent);
 	}
 
-	null_links = (opts->min_null_count > 0);
+	sent->null_links = (opts->min_null_count > 0);
 
 	has_conjunction = sentence_contains_conjunction(sent);
 	set_connector_length_limits(sent, opts);
@@ -306,7 +309,6 @@ void prepare_to_parse(Sentence sent, Parse_Options opts)
 		if (verbosity > 2) {
 			printf("\nAfter conjunction pruning:\n");
 			print_disjunct_counts(sent);
-			print_statistics();
 		}
 		print_time(opts, "Done conjunction pruning");
 		build_conjunction_tables(sent);
