@@ -19,7 +19,6 @@
 
 #define MAX_STRIP 10
 
-int post_quote[MAX_SENTENCE];
 /*static char * strip_left[] = {"(", "$", "``", NULL}; */
 /*static char * strip_right[] = {")", "%", ",", ".", ":", ";", "?", "!", "''", "'", "'s", NULL};*/
 
@@ -140,14 +139,13 @@ static int issue_sentence_word(Sentence sent, const char * s)
 {
 	if (*s == '\0') return TRUE;
 	if (strlen(s) > MAX_WORD) {
-		lperror(SEPARATE,
-				". The word \"%s\" is too long.\n"
+		prt_error("Error separating sentence. The word \"%s\" is too long.\n"
 				"A word can have a maximum of %d characters.\n", s, MAX_WORD);
 		return FALSE;
 	}
 
 	if (sent->length == MAX_SENTENCE) {
-		lperror(SEPARATE, ". The sentence has too many words.\n");
+		prt_error("Error separating sentence. The sentence has too many words.\n");
 		return FALSE;
 	}
 
@@ -229,10 +227,10 @@ static int downcase_is_in_dict(Dictionary dict, char * word)
 
 	nbh = mbtowc (&c, word, 4);
 	c = towlower(c);
-	nbl = wctomb(low, c);
+	nbl = wctomb_check(low, c);
 	if (nbh != nbl)
 	{
-		fprintf(stderr, "Error: can't downcase multi-byte string!\n");
+		prt_error("Error: can't downcase multi-byte string!\n");
 		return FALSE;
 	}
 
@@ -246,13 +244,15 @@ static int downcase_is_in_dict(Dictionary dict, char * word)
 	return rc; 
 }
 
+/**
+ * w points to a string, wend points to the char one after the end.  The
+ * "word" w contains no blanks.  This function splits up the word if
+ * necessary, and calls "issue_sentence_word()" on each of the resulting
+ * parts.  The process is described above.  returns TRUE of OK, FALSE if
+ * too many punctuation marks 
+ */
 static int separate_word(Sentence sent, char *w, char *wend, int is_first_word, int quote_found)
 {
-	/* w points to a string, wend points to the char one after the end.  The
-	 * "word" w contains no blanks.  This function splits up the word if
-	 * necessary, and calls "issue_sentence_word()" on each of the resulting
-	 * parts.  The process is described above.  returns TRUE of OK, FALSE if
-	 * too many punctuation marks */
 	int i, j, k, l, len;
 	int r_strippable=0, l_strippable=0;
 	int s_strippable=0, p_strippable=0;
@@ -438,12 +438,12 @@ static int separate_word(Sentence sent, char *w, char *wend, int is_first_word,
 
 	/*
 	if (n_stripped == MAX_STRIP) {
-		lperror(SEPARATE,
-				".\n\"%s\" is followed by too many punctuation marks.\n", word);
+		prt_error("Error separating sentence.\n"
+		          "\"%s\" is followed by too many punctuation marks.\n", word);
 		return FALSE;
 	} */
 
-	if (quote_found==1) post_quote[sent->length]=1;
+	if (quote_found == TRUE) sent->post_quote[sent->length]=1;
 
 	if (!issue_sentence_word(sent, word)) return FALSE;
 
@@ -492,14 +492,16 @@ int separate_sentence(char * s, Sentence sent)
 	int i, is_first, quote_found;
 	Dictionary dict = sent->dict;
 
-	for(i=0; i<MAX_SENTENCE; i++) post_quote[i]=0;
+	for(i=0; i<MAX_SENTENCE; i++) sent->post_quote[i] = 0;
 	sent->length = 0;
 
 	if (dict->left_wall_defined)
 		if (!issue_sentence_word(sent, LEFT_WALL_WORD)) return FALSE;
 
+#ifndef __CYGWIN__
 	/* Reset the multibyte shift state to the initial state */
 	mbtowc(NULL, NULL, 4);
+#endif
 
 	is_first = TRUE;
 	for(;;) 
@@ -514,7 +516,7 @@ int separate_sentence(char * s, Sentence sent)
 		while (iswspace(c) || (c == '\"'))
 		{
 			s += nb;
-			if(*s == '\"') quote_found=TRUE;
+			if (*s == '\"') quote_found = TRUE;
 			nb = mbtowc(&c, s, 4);
 			if (0 > nb) goto failure;
 		}
@@ -544,12 +546,14 @@ int separate_sentence(char * s, Sentence sent)
 
 failure:
 #ifndef _WIN32
-	lperror(CHARSET, "%s\n", nl_langinfo(CODESET));
+	prt_error("Unable to process UTF8 input string in current locale %s\n",
+		nl_langinfo(CODESET));
 #endif
 	return FALSE;
 }
 
-static int special_string(Sentence sent, int i, const char * s) {
+static int special_string(Sentence sent, int i, const char * s)
+{
 	X_node * e;
 	if (boolean_dictionary_lookup(sent->dict, s)) {
 		sent->word[i].x = build_word_expressions(sent, s);
@@ -558,8 +562,9 @@ static int special_string(Sentence sent, int i, const char * s) {
 		}
 		return TRUE;
 	} else {
-		lperror(BUILDEXPR, ".\n To process this sentence your dictionary "
-				"needs the word \"%s\".\n", s);
+		prt_error("Error: Could not build sentence expressions.\n"
+		          "To process this sentence your dictionary "
+ 		          "needs the word \"%s\".\n", s);
 		return FALSE;
 	}
 }
@@ -609,8 +614,9 @@ static int guessed_string(Sentence sent, int i, const char * s, const char * typ
 		  return TRUE;
 
 	} else {
-		lperror(BUILDEXPR, ".\n To process this sentence your dictionary "
-				"needs the word \"%s\".\n", type);
+		prt_error("Could not build sentence expressions.\n"
+		          "To process this sentence your dictionary "
+		          "needs the word \"%s\".\n", type);
 		return FALSE;
 	}
 }
@@ -744,7 +750,9 @@ int build_sentence_expressions(Sentence sent)
 	 */
 	for (i=0; i<sent->length; i++)
 	{
-		if (! (i==first_word || (i>0 && strcmp(":", sent->word[i-1].string)==0) || post_quote[i]==1) ) continue;
+		if (! (i == first_word ||
+		      (i > 0 && strcmp(":", sent->word[i-1].string)==0) || 
+		       sent->post_quote[i] == 1)) continue;
 		s = sent->word[i].string;
 
 		if (is_utf8_upper(s))
@@ -820,7 +828,7 @@ int sentence_in_dictionary(Sentence sent)
 		}
 	}
 	if (!ok_so_far) {
-		lperror(NOTINDICT, "\n%s\n", temp);
+		prt_error("Error: Sentence not in dictionary\n%s\n", temp);
 	}
 	return ok_so_far;
 }
